#ifndef XPLODE_H
#define XPLODE_H

// Qt includes
#include <qapp.h>
#include <qmainwindow.h>
#include <qaction.h>
#include <qtabwidget.h>
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qstatusbar.h>
#include <qwhatsthis.h>
#include <qpopupmenu.h>
#include <qaccel.h>
#include <qtoolbutton.h>
#include <qmsgbox.h>
#include <qfiledialog.h>
#include <qstring.h>
#include <qpixmap.h>
#include <qaccel.h>
#include <qsplitter.h>
#include <qmap.h>
#include <qinputdialog.h>
#include <qfontdialog.h>
#include <qbutton.h>
#include <qvaluelist.h>
#include <qlistbox.h>
#include <qprocess.h>
#include <qtextbrowser.h>

// Xplode includes
#include "xplodedoc.h"
#include "xplodeeditor.h"
#include "xplodelistener.h"
#include "xplodeconfig.h"

// The main class
class XplodeApp : public QMainWindow
{
	Q_OBJECT

public:
	XplodeApp();
	~XplodeApp();

	void openDocumentFile(const char *file = 0);

protected:
	bool queryExit();
	void closeEvent(QCloseEvent *e);

private slots:
	void slotFileNew();
	void slotFileOpen();
	void slotFileSave();
	void slotFileSaveAs();
	void slotFileClose();
	void slotFileQuit();

	void slotEditUndo();
	void slotEditCut();
	void slotEditCopy();
	void slotEditPaste();
	void slotEditSelectAll();
	void slotEditSearch();
	void slotEditSearchNext();
	void slotEditReplace();

	void slotPrefMzplasm();
	void slotPrefVrml();
	void slotPrefSvg();
	void slotPrefFlash();
	void slotPrefEditFont();
	void slotPrefListenerFont();
	void slotPrefSave();

	void slotPlasmExport();
	void slotPlasmVrml();
	void slotPlasmConnect();
	void slotPlasmExpr();
	void slotPlasmBuffer();

	void slotHelpHelp();
	void slotHelpAbout();
	void slotHelpAboutQtLib();

	void slotStartEvaluation();
	void slotStopEvaluation();
	void slotIncrementEvaluation();

	void slotKillEvaluation();
	void slotAddExported(QString filename);
	void slotViewExported(QListBoxItem* item);
	void slotFileModified(XplodeDoc* editor);
	void slotFileSaved(XplodeDoc* editor);
	void slotShowCursorPosition(QString);
private:
	void initDefaults();
	void initActions();
	void initMenuBar();
	void initToolBar();
	void initStatusBar();
	void initView();

	QPopupMenu *pFileMenu;
	QPopupMenu *pEditMenu;
	QPopupMenu *pPrefMenu;
	QPopupMenu *pPrefViewersMenu;
	QPopupMenu *pPrefFontMenu;
	QPopupMenu *pPlasmMenu;
	QPopupMenu *pHelpMenu;

	QToolBar *xplodeToolbar;

	QSplitter *pWorkspace, *pDock;
	QTabWidget *pTabbedWorkspace;
	QListBox *pExported;

	int untitledCount;

	QPtrList<XplodeDoc> *pDocList;

	QAction *fileNew;
	QAction *fileOpen;
	QAction *fileSave;
	QAction *fileSaveAs;
	QAction *fileClose;
	QAction *fileQuit;

	QAction *editUndo;
	QAction *editCut;
	QAction *editCopy;
	QAction *editPaste;
	QAction *editSelectAll;
	QAction *editSearch;
	QAction *editSearchNext;
	QAction *editReplace;

	QAction *prefMzplasm;
	QAction *prefVrml;
	QAction *prefSvg;
	QAction *prefFlash;
	QAction *prefEditFont;
	QAction *prefListenerFont;
	QAction *prefSave;

	QAction *plasmLibrary;
	QAction *plasmExport;
	QAction *plasmVrml;
	QAction *plasmConnect;
	QAction *plasmExpr;
	QAction *plasmBuffer;

	QAction *helpHelp;
	QAction *helpAboutApp;
	QAction *helpAboutQt;

	XplodeListener *listener;
	XplodeConfig *config;

	QLabel *statusText, *cursorPos;
	QPushButton *abortEval;
	QProgressBar *evalProgress;
	QTimer *evalTimer;
	bool fwdProgress;

	QString target, replace;
	QStringList *oldTargets, *oldReplaces;
	bool bRegexp, bCase, bBack, bCursor, bAll;
};

#endif
